@@ -8,8 +8,8 @@ package IO::Async::Loop::Poll;
 use strict;
 use warnings;
 
-our $VERSION = '0.32';
-use constant API_VERSION => '0.24';
+our $VERSION = '0.33';
+use constant API_VERSION => '0.33';
 
 use base qw( IO::Async::Loop );
 
@@ -245,7 +245,8 @@ sub unwatch_io
 
    $self->__unwatch_io( %params );
 
-   my $poll = $self->{poll};
+   # Guard for global destruction
+   my $poll = $self->{poll} or return;
 
    my $handle = $params{handle};
 
